#!/cygdrive/c/Perl/bin/perl.exe -w
#
###########################################################
# tm2xls.pl -
#   Parses a TECS XML file into a Microsoft Excel XLS file.
#
# Version: 1.0
#
# Written by: Justin Zipkin
#             justin.zipkin@db.com
#
# Modified:   02.12.2005 (jz)
#
# Revision Notes:
# 1.0  - Initial Release
#
# As an after-sight to creating tm2csv.pl, and the entire
# TECS system, we realized that it would be useful to have
# a very simple way to determine what jobs are good
# candidates to move off of TaskMan first.  Although
# factors for determining what may be a good candidate are
# somewhat abstract, this script will consolidate some use-
# ful numbers for you.  Specifically, the number of tasks
# that one task waits on, that wait on it, and if there are
# any dead dependencies that appear as if they will never
# run.  This script is made to be run against an XML file
# generated by TECS.
#
###########################################################

use strict;
use XML::Parser;
use Spreadsheet::WriteExcel;

###########################################################
### FUNCTION PROTOTYPES
###########################################################
sub ParseXML($);
sub WriteXLS($$);
sub DisplayUsage();

###########################################################
### START CODE
###########################################################

if ($#ARGV!=1) {
  &DisplayUsage;
  exit(1);
}

my ($xml,$xls)=@ARGV;

# Call WriteXLS for the passed parameters
WriteXLS($xml,$xls);


###########################################################
# START SUBROUTINES
###########################################################

#### 
# ParseXML($) - Parse our XML file
# 
# Takes: A string containing a filename containing the
#        TaskMan XML
# Returns: An array of pointers to arrays to tasks...
#          Each array element will have a reference that
#          points to an array of 12 scalars, the associated
#          scalar values represent the following task
#          information:
#            1) Task Name
#            2) Filename
#            3) Task ID
#            4) Type
#            5) Dependency Failure
#            6) Predecessors
#            7) Postdecessors
#            8) Wait Ons
#            9) Frequency
#            10) Period
#            11) Start
#            12) Stop
#
# This sub is kind of ugly, and has numbers all over the
# place that happen to be absolutely necessary.  All I
# can say is that with a properly formatted XML file
# this should work.  If something is a bit awry the
# results will be unanticpatable (is that a word?).  It
# may be worthwile to do a DTD validation before going
# through all of this effort, but that is for another day.
#
###
sub ParseXML($) {
  # Define some variables
  my $date;
  my $count;
  my @taskpostdecessors;
  my (@tasks, $task, $special_task, $wait_on, $check_task, $wait_on_good);
  my @specialtasks=('RANBASE EOD COMPLETE', 'RANBASE BATCH 2 STARTED', 'SOURCE 0/6/7 COMPLETE');
  
  # Create a parser and load it with our XML file
  my $parser=new XML::Parser(Style=>'Tree');;
  my $file=shift(@_);
  my @tree=$parser->parsefile($file);
  
  # Populate the date from the XML file
  $date=$tree[0][1][0]{date};

  # Set our count to be at four, because of the structure
  # of our XML file, a properly formatted file (conforming
  # to the DTD) should work in this scenario.
  $count=4;

  # Loop as long as there is stuff to loop
  while (defined($tree[0][1][$count])) {
    # If we come across a task, this is the only time we care
    if ($tree[0][1][$count-1] eq 'task') {
      # Define some localized variables
      my ($name, $type, $wait_on_count);
      my (@wait_on);
      my ($frequency, $period, $start, $stop, $late_alert)='';
      my $filename = $tree[0][1][$count][0]{'filename'};
      my $task_id = $tree[0][1][$count][0]{'id'};
      my @taskinfo;
      
      # Now to the heart, we start at index 3 and loop, incrementing by four every time... why? 
      # That just happens to be how our XML document looks.
      for (my $array_index=3; defined($tree[0][1][$count][$array_index])==1; $array_index=$array_index+4) {
        # If it's a name, store it
        if ($tree[0][1][$count][$array_index] eq 'name'){
          $name=$tree[0][1][$count][$array_index+1][2];}
        # If it's a type, store it
        elsif ($tree[0][1][$count][$array_index] eq 'type'){
          $type=$tree[0][1][$count][$array_index+1][2];}
        # If it's a schedule, dive down another level
        elsif ($tree[0][1][$count][$array_index] eq 'schedule') {
          # Now we start looping again, starting at an index of three and incrementing by four.
          # This time we record any frequencies, periods, starts, stops, wait-ons (in an array),
          # and late-alerts
          for (my $sub_array_index=3; defined($tree[0][1][$count][$array_index+1][$sub_array_index])==1; $sub_array_index=$sub_array_index+4) {
            if ($tree[0][1][$count][$array_index+1][$sub_array_index] eq 'frequency'){
              $frequency=$tree[0][1][$count][$array_index+1][$sub_array_index+1][2];
            }
            elsif ($tree[0][1][$count][$array_index+1][$sub_array_index] eq 'period'){
              $period=$tree[0][1][$count][$array_index+1][$sub_array_index+1][2];
            }
            elsif ($tree[0][1][$count][$array_index+1][$sub_array_index] eq 'start'){
              $start=$tree[0][1][$count][$array_index+1][$sub_array_index+1][2];
            }
            elsif ($tree[0][1][$count][$array_index+1][$sub_array_index] eq 'stop'){
              $stop=$tree[0][1][$count][$array_index+1][$sub_array_index+1][2];
            }
            elsif ($tree[0][1][$count][$array_index+1][$sub_array_index] eq 'wait-on'){
              push(@wait_on,$tree[0][1][$count][$array_index+1][$sub_array_index+1][2]);
            }
            elsif ($tree[0][1][$count][$array_index+1][$sub_array_index] eq 'late-alert'){
              $late_alert=$tree[0][1][$count][$array_index+1][$sub_array_index+1][2];
            }
          }
        }
      }
      
      $wait_on_count=@wait_on;
      
      # 1) Name
      # 2) Filename
      # 3) Task ID
      # 4) Type
      # 5) Dependency Failure
      # 6) Predecessors
      # 7) Postdecessors
      # 8) Wait Ons
      # 9) Frequency
      # 10) Period
      # 11) Start
      # 12) Stop
      @taskinfo=($name,$filename,$task_id,$type,'',$wait_on_count,'',join(', ',@wait_on),$frequency,$period,$start,$stop);
      push (@tasks,\@taskinfo);
    }
    $count++;
  }
  
  # Identify tasks that look like they will not run, this
  # little procedure can be a pain to follow, hopefully the
  # pseudo-code will be more of a help than a hinderence.
  foreach $task (@tasks) {
    # For each task, take the list of wait-ons
    foreach $wait_on (split(/,\s*/,$$task[7])) {
      # Since we're in here we are going to store the wait-
      # on, I know this doesn't make sense here, but it
      # will save time and it's something we want to know
      # later.
      push (@taskpostdecessors, $wait_on);
      
      # Assume that the wait-on doesn't exist
      $wait_on_good=0;
      foreach $special_task (@specialtasks) {
        # Check to see if the wait on is a special task,
        # if it is then don't worry about it
        if ($wait_on eq $special_task) {
          $wait_on_good=1;
        }
      }
      # If we're not set yet (presumably by
      # checking if it's a special task)...
      if ($wait_on_good!=1) {
        # Then we have to iterate through all
        # off the tasks to see if it exists.
        foreach $check_task (@tasks) {
          # If we find it in the list, then we
          # are good!
          if ($wait_on eq $$check_task[0]) {
            $wait_on_good=1;
          }
        }
      }
      # If we haven't found anything then the
      # wait-on is no good, populate the failed
      # dependency field with this wait-on name.
      if ($wait_on_good!=1) {
        $$task[4]=$wait_on;
      }
    }
  }
  
  # Populate the number of postdecessors for each task,
  # this is where we use that array @taskpsotdecessors 
  # that we populated in the last loop.
  foreach $task (@tasks) {
    $$task[6]=scalar(map(/^$$task[0]$/,@taskpostdecessors));
  }
  
  return(@tasks);
}

#### 
# WriteXLS($$) - Parse our XML file
# 
# Takes: A string containing a filename containing the
#        TaskMan XML and a string containing the output
#        XLS file.
# Returns: The XLS file that it output.
#
###
sub WriteXLS($$) {
  my ($infile, $outfile)=@_;
  my (@tasks, $task, $row);
  my ($workbook, $sheet_tasklist, $format_pointer);
  my ($format_headers, $format_good, $format_bad);
  
  @tasks=ParseXML($infile);
  
  # Create a new Excel workbook called perl.xls
  $workbook=Spreadsheet::WriteExcel->new($outfile);
  
  # Format definition for the headers
  $format_headers=$workbook->add_format();
  $format_headers->set_font('Times New Roman');
  $format_headers->set_size('10');
  $format_headers->set_bold();
  $format_headers->set_underline();
  
  # Format definition for tasks that look "good"
  $format_good=$workbook->add_format();
  $format_good->set_font('Times New Roman');
  $format_good->set_size('8');
  $format_good->set_color('green');
  
  # Format definition for tasks taht look "bad"
  $format_bad=$workbook->add_format();
  $format_bad->set_font('Times New Roman');
  $format_bad->set_size('8');
  $format_bad->set_color('red');
  
  # Create a new worksheet and configure some stuff
  $sheet_tasklist=$workbook->add_worksheet("01-12-2005");
  $sheet_tasklist->repeat_rows(0);
  $sheet_tasklist->set_landscape();
  $sheet_tasklist->repeat_rows(0,0);
  $sheet_tasklist->repeat_columns(0,0);
  $sheet_tasklist->print_area('A:H');
  $sheet_tasklist->set_row(0,18,$format_headers);
  $sheet_tasklist->set_column('A:A',33);
  $sheet_tasklist->set_column('B:B',20);
  $sheet_tasklist->set_column('C:C',6);
  $sheet_tasklist->set_column('D:D',24);
  $sheet_tasklist->set_column('E:E',20);
  $sheet_tasklist->set_column('F:G',11);
  $sheet_tasklist->set_column('H:H',93);
  $sheet_tasklist->write(0,0,"Name");
  $sheet_tasklist->write(0,1,"Filename");
  $sheet_tasklist->write(0,2,"Task ID");
  $sheet_tasklist->write(0,3,"Type");
  $sheet_tasklist->write(0,4,"Dependency Failure");
  $sheet_tasklist->write(0,5,"Predecessors");
  $sheet_tasklist->write(0,6,"Postdecessors");
  $sheet_tasklist->write(0,7,"Wait-Ons");
  
  # Start a row 1 and populate the fields with data
  $row=1;
  
  foreach $task (@tasks) {
    if ($$task[4] eq '') {
      $format_pointer=\$format_good;
    }
    else {
      $format_pointer=\$format_bad;
    }
    for (my $j=0; $j<=7; $j++) {
      $sheet_tasklist->write($row,$j,$$task[$j],$$format_pointer);
    }
    $row++;
  }
  
  return($outfile);
}

#### 
# DisplayUsage() - Display script usage
# 
# Takes: A string containing a filename containing the
#        TaskMan XML and a string containing the output
#        XLS file.
# Returns: The XLS file that it output.
#
###
sub DisplayUsage() {
  print "Usage: $0 InputFile OutputFile\n\n";
  print "InputFile    File to read data from in TECS XML format\n";
  print "OutputFile   File to output binary data to, probably something.xls\n";
}
