#!/bin/perl -wT
###########################################################
# tm2csv.pl -
#   Parse a TaskMan export file and dump it into a CSV
#
# Version: 1.20
#
# Written by: Justin Zipkin
#             justin.zipkin@db.com
#
# Modified:   31.10.2005 (jz)
#
# Revision Notes:
# 1.0  - Initial Release
# 1.01 - Fixed warning when no parameter was passed bug
# 1.10 - Added support for oddly formatted file types
#        (task.xxxx.name vs task.xxxx.common.name)
# 1.11 - Fixed a bug for non-scheduled, oddly formatted
#        tasks
# 1.20 - Added support for command line switches and the
#        ability to define a custom string of files to
#        process.
#
###########################################################

use strict;
use warnings;
use Getopt::Std;

###########################################################
### FUNCTION PROTOTYPES
###########################################################
sub ReadFile($);
sub GetNames(@);
sub GetTypes(@);
sub GetSchedules(@);
sub GetUniqueTypes(@);         # Never used, but a neat sub
sub RemoveDuplicates(@);
sub OutputInfo($%);
sub OutputInfoMulti($%);
sub DisplayUsage();

###########################################################
### VARIABLE DECLARATIONS
###########################################################
my($line,$key,$file);
my(@exportfile);
my(%names,%types,%schedules,@uniquetypes);
my(%jobinfo);
my(%args)=();

###########################################################
### START CODE
###########################################################

getopts("hp:",\%args);

# Display help
if (exists($args{h})) {
  &DisplayUsage;
  exit(0);
}

# Process multiple files based on regexp
if (exists($args{p})) {
  my @files;

  print "File,Task,Name,Type,Scheduled\n";
  
  # Get a list of all files ending in .export
  opendir(DIR, ".");
  @files = grep(/$args{p}/,readdir(DIR));
  closedir(DIR);
  
  # For each export file, run through all of the entries
  # in the same way as above
  foreach $file (@files) {
    %jobinfo=();
    @exportfile=ReadFile($file);
    %names=GetNames(@exportfile);
    %types=GetTypes(@exportfile);
    %schedules=GetSchedules(@exportfile);

    while (($key) = each(%names)){
      push(@{$jobinfo{$key}}, $names{$key});
      push(@{$jobinfo{$key}}, $types{$key});
      push(@{$jobinfo{$key}}, $schedules{$key});
    }
    OutputInfo($file,%jobinfo);
  }
  exit(0);
}

# Process a single file off the command line
if ($#ARGV==0) {
  $file=$ARGV[0];
  
  # Read the file into memory
  @exportfile=ReadFile($file);

  # Get a hash of task names
  %names=GetNames(@exportfile);
  
  # Get a hash of task types
  %types=GetTypes(@exportfile);
  
  # Get a hash of task schedules
  %schedules=GetSchedules(@exportfile);

  # Cool function, should implement some command line
  # switches to utilize it
  #@uniquetypes=GetUniqueTypes(@exportfile);

  # Header Row
  print "File,Task,Name,Type,Scheduled\n";
  
  # Here we assume any running task has a name.  We
  # loop through the names hash and for every key
  # (task number) we put populate a hash (jobinfo)
  # with the associated names, types, and schedules.
  while (($key) = each(%names)){
    push(@{$jobinfo{$key}}, $names{$key});
    push(@{$jobinfo{$key}}, $types{$key});
    push(@{$jobinfo{$key}}, $schedules{$key});
  }
  
  # Output the fully populated hash
  OutputInfo($file,%jobinfo);
  exit(0);
}

# Syntax looks funny, just display help
else {
  &DisplayUsage;
  exit(0);
}

###########################################################
# START SUBROUTINES
###########################################################

#### 
# ReadFile($) - Read a file into an array
# 
# Takes: A scalar giving a standard pathname
# Returns: An array containing the file data
###
sub ReadFile($) {
  my(@export);
  open EXPORTFILE, $_[0];
  @export=<EXPORTFILE>;
  close EXPORTFILE;
  return (@export);
}

###
# GetNames(@) - Get the task names out of an array
#
# Takes: An array composed of a TaskMan export file
# Returns: A hash relating task ID's to their Names
###
sub GetNames(@) {
  my(@tasknames,@weirdtasknames);
  my($line,$task,$name);
  my(%tasknameshash);
  @tasknames=grep(/task\.....\.common\.name=/, @_);
  @weirdtasknames=grep(/task\.....\.name=/, @_);
  chomp(@tasknames);
  chomp(@weirdtasknames);
  foreach $line (@tasknames) {
    $task=substr($line,5,4);
    $name=substr($line,22);
    $tasknameshash{$task}=$name;
  }
  foreach $line (@weirdtasknames) {
    $task=substr($line,5,4);
    $name=substr($line,15);
    $tasknameshash{$task}=$name;
  }
  return(%tasknameshash);
}

###
# GetTypes(@) - Get the task types out of an array
#
# Takes: An array composed of a TaskMan export file
# Returns: A hash relating task ID's to their Types
###
sub GetTypes(@) {
  my(@tasktypes,@weirdtasktypes);
  my($line,$task,$type);
  my(%tasktypehash);
  @tasktypes=grep(/task\.....\.common\.type=/, @_);
  @weirdtasktypes=grep(/task\.....\.type=/, @_);
  #@tasktypes=grep(/task\.....\.type=/,@_);
  chomp(@tasktypes);
  chomp(@weirdtasktypes);
  foreach $line (@tasktypes) {
    $task=substr($line,5,4);
    $type=substr($line,22);
    $tasktypehash{$task}=$type;
  }
  foreach $line (@weirdtasktypes) {
    $task=substr($line,5,4);
    $type=substr($line,15);
    $tasktypehash{$task}=$type;
  }
  return(%tasktypehash);
}

###
# GetSchedules(@) - Get the tasks scheduling status
#                   out of an array
#
# Takes: An array composed of a TaskMan export file
# Returns: A hash relating task ID's to whether they
#          are scheduled or not.
###
sub GetSchedules(@) {
  my(@tasks, @tasksweird, @tasksched,@weirdtasksched, @runningtasks);
  my($line,$task);
  my(%taskschedules);
  @tasks=grep(/task\.....\.common\.name=/, @_);
  @tasksweird=grep(/task\.....\.name=/, @_);
  @tasksched=grep(/task\.....\.common\.schedule\./, @_);
  @weirdtasksched=grep(/task\.....\.schedule\./, @_);
  
  # First set all tasks (with names) to No
  foreach $line (@tasks) {
    $task=substr($line,5,4);
    $taskschedules{$task}="No";
  }
  
  # Do the same for tasks that use a non-standard
  # naming convention
  foreach $line (@tasksweird) {
    $task=substr($line,5,4);
    $taskschedules{$task}="No";
  }
  
  # Next set all tasks with a schedule field to Yes
  foreach $line (@tasksched) {
    $task=substr($line,5,4);
    $taskschedules{$task}="Yes";
  }
  
  # Next set all tasks with a schedule field to Yes
  # that use a non-standard naming convention
  foreach $line (@weirdtasksched) {
    $task=substr($line,5,4);
    $taskschedules{$task}="Yes";
  }
    
  return(%taskschedules);
}

###
# GetUniqueTypes(@) - Get the unique types out of an
#                     array
#
# Takes: An array composed of a TaskMan export file
# Returns: An array containing a list of the types
#          in a file
###
sub GetUniqueTypes(@) {
  my(@tasktypes,@uniquetasktypes);
  my($line,$counter);
  $counter=0;
  @tasktypes=GetTypes(@_);
  foreach $line (@tasktypes) {
    $counter++;
    if (!($counter%2)) {
      push(@uniquetasktypes,$line);
    }
  }
  RemoveDuplicates(sort(@uniquetasktypes));
}

###
# RemoveDuplicates(@) - Remove duplicates from a sorted
#                       array
#
# Takes: A sorted array
# Returns: An array with duplicates removed
###
sub RemoveDuplicates(@) {
  my($p);
  $p="bOgUsDaTa";
  grep($_ ne $p && ($p=$_,1),@_);
}

###
# OutputInfo(%) - Output the formatted TaskMan data
#
# Takes: A hash of array references
# Returns: 
###
sub OutputInfo($%) {
  my($line,$towrite,$file);
  my(%data)=();
  $file=shift(@_);
  %data=@_;
  foreach $line (keys %data) {
    $towrite="$file,$line,@{$data{$line}}[0],@{$data{$line}}[1],@{$data{$line}}[2]\n";
    $towrite=~s/\x0D//g;
    print $towrite;
  }
}

sub DisplayUsage() {
  print "Usage: $0 [-h] [[-p regexp] | Filename]\n\n";
  print "-h           Display this message\n";
  print "-p [regexp]  A regular expression pattern defining a list of files to process.\n";
  print "             For example $0 -p '.export\$' will process all files\n";
  print "             ending in .export\n";
  print "Filename     A single filename to process.\n";
}

###########################################################
# END SUBROUTINES
###########################################################
